#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or(at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2000-2004 Free Software Foundation
#
# FILE:
# Introspection.py
#
# DESCRIPTION:
#
# NOTES:
#

__all__ = ['Introspection']

import string
from string import lower, join, split
import sys

from gnue.common.apps import GDebug, GConfig
from gnue.common.apps import GDebug, GConfig
from gnue.common.datasources import GIntrospection

class Introspection(GIntrospection.Introspection):

  # list of the types of Schema objects this driver provides
  types =  [ ('user_table',   _('User Tables'),1),
             ('user_view',    _('User Views'),1),
             ('user_synonym', _('User Synonyms'),1),
             ('all_table',    _('System Tables'),1),
             ('all_view',     _('System Views'),1),
             ('all_synonym',  _('System Synonyms'),1) ]

  #
  # TODO: This is a quick hack to get this class
  # TODO: into the new-style schema format.
  # TODO: getSchema* should be merged into find()
  #
  def find(self, name=None, type=None):
    if name is None:
      return self.getSchemaList(type)
    else:
      rs = self.getSchemaByName(name, type)
      if rs:
        return [rs]
      else:
        return None


  # TODO: Merge into find()
  # Return a list of Schema objects
  # Return a list of Schema objects
  def getSchemaList(self, type=None):

    where_user = ""
    if type == None:
      where_type = ['TABLE', 'VIEW', 'SYNONYM']
    else:
      scope, type = string.split(type,'_')
      where_type = [string.upper(type)]
      if scope == 'user':
        where_user = " AND OWNER = USER"


    statement = \
      "select owner||'.'||table_name||'.'||table_type full_name, \n" + \
      "  decode(owner,user,null,owner||'.')||table_name table_name, \n" + \
      "  decode(owner,user,'user_','all_')||lower(table_type) table_type \n" + \
      "  from all_catalog where table_type in ('%s') %s \n" \
              % (string.join(where_type,"','"), where_user) + \
      "  order by table_name "

    GDebug.printMesg(5,statement)

    cursor = self._connection.native.cursor()
    cursor.execute(statement)

    list = []
    for rs in cursor.fetchall():
      list.append(GIntrospection.Schema(attrs={'id':rs[0], 'name':string.lower(rs[1]),
                         'type':rs[2]},
                         getChildSchema=self.__getFieldSchema))

    cursor.close()
    return list


  # Find a schema object with specified name
  def getSchemaByName(self, name, type=None):

    spl = string.split(string.upper(name),'.')
    where = "TABLE_NAME='%s'" % spl[-1]
    if len(spl) > 1:
      where += " AND OWNER='%s'" % spl[-2]

    statement = \
      "select owner||'.'||table_name||'.'||table_type full_name, \n" + \
      "  decode(owner,user,null,owner||'.')||table_name table_name, \n" + \
      "  decode(owner,user,'user_','all_')||lower(table_type) table_type \n" + \
      "  from all_catalog where %s " \
              % (where)

    GDebug.printMesg(5,statement)

    cursor = self._connection.native.cursor()
    cursor.execute(statement)

    list = []
    rs = cursor.fetchone()
    if rs:
      rv = GIntrospection.Schema(attrs={'id':rs[0], 'name':string.lower(rs[1]),
                         'type':rs[2]},
                         getChildSchema=self.__getFieldSchema)
    else:
      rv = None

    cursor.close()
    return rv


  # Get fields for a table
  def __getFieldSchema(self, parent):

    owner, name, type = string.split(parent.id,'.')

    cursor = self._connection.native.cursor()

    if type == 'SYNONYM':
      statement = "select table_owner, table_name, " + \
                  "decode(db_link,null,'','@'||db_link) name " + \
                  "from all_synonyms " + \
                  "where owner = '%s' and synonym_name='%s'" % (owner, name)

      GDebug.printMesg(5,statement)

      cursor.execute(statement)
      rs = cursor.fetchone()
      owner, name, link = rs
      if link is None:
        link = ""
    else:
      link = ""

    statement = \
       "select owner||'.'||table_name||'.'||column_name||'.%s', " % (link) + \
       "column_name, data_type, nullable, data_length, data_scale, data_precision " + \
       "from all_tab_columns%s " % (link) + \
       "where owner = '%s' and table_name = '%s' " % (owner, name) + \
       "order by column_id"

    GDebug.printMesg(5,statement)

    cursor.execute(statement)

    list = []
    for rs in cursor.fetchall():

      attrs={'id': rs[0], 'name': string.lower(rs[1]),
             'type':'field', 'nativetype': rs[2],
             'required': rs[3] == 'N'}

      if rs[2] in ('NUMBER',):
        attrs['precision'] = int(rs[5])
        attrs['datatype'] = 'number'
        attrs['length'] = int(rs[6])
      elif rs[2] in ('DATE',):
        attrs['datatype'] = 'date'
      else:
        attrs['datatype'] = 'text'
        if int(rs[4]):
          attrs['length'] = int(rs[4])

      list.append(GIntrospection.Schema(attrs=attrs))

    cursor.close()
    return tuple(list)

